(defun muut (f x0 delta)
  (- (funcall f (+ x0 delta)) (funcall f x0)))

(defun gen/filter (pred list)
  (labels ((iter (list collect)
	     (cond ((null list)
		    collect)
		   ((funcall pred (first list))
		    (iter (rest list) (cons (first list) collect)))
		   (t (iter (rest list) collect)))))
    (reverse (iter list nil))))

(defun gen/any (pred list)
  (cond ((null list) nil)
	((funcall pred (car list)) t)
	(t (gen/any pred (cdr list)))))

(defun gen/none (pred list)
  (not (gen/any pred list)))

(defun gen/all (pred list)
  (not (gen/any (lambda (x) (not (funcall pred x))) list)))

(defun gen/some (pred list)
  (gen/any (lambda (x) (not (funcall pred x))) list))

(defun gen/onep (x)
  (eq 1 x))



(defun calc/variable? (x)
  (symbolp x))
(defun calc/same-variable? (x y)
  (and (calc/variable? x) (calc/variable? y) (eq x y)))


(defun calc/defun (f args)
  (cons f args))

(defun calc/do (f &rest var)
  (eval `(let ,var
	   (funcall ,(car f) ,@(cdr f)))))

(defun calc/sum (&rest args)
  (if (> (length args) 2)
      (calc/sum (car args) (calc/sum (cdr args)))
      (calc/defun #'+ args)))
(defun calc/product (&rest args)
  (calc/defun #'* args))
(defun calc/sum? (f)
  (and (listp f) (eq (car f) #'+)))
(defun calc/product? (f)
  (and (listp f) (eq (car f) #'*)))


(defun calc/deriv (exp var)
  (calc/simplify
   (cond ((numberp exp) 0)
	 ((calc/variable? exp)
	  (if (calc/same-variable? exp var) 1 0))
	 ((calc/sum? exp)
	  (apply #'calc/sum
		(map 'list
		     (lambda (x) (calc/deriv x var))
		     (cdr exp))))
	 ((calc/product? exp)
	  (if (= (length exp) 3)
	      (calc/sum
	       (calc/product
		(calc/deriv (second exp) var)
		(third exp))
	       (calc/product
		(second exp)
		(calc/deriv (third exp) var)))
	      
	      (calc/deriv (calc/product ;Pole selles kindel
			   (calc/deriv
			    (calc/product
			     (second exp)
			     (third exp))
			    var)
			   (cdddr exp))
			  var)))
	 (t (error "unknown expression type -- DERIV")))))

(defun calc/simplify (exp)
  (cond ((calc/sum? exp)
	 (let ((step
		 (gen/filter (lambda (x) (not (eq 0 x)))
			     (map 'list #'calc/simplify (rest exp)))))
	   (cond ((null step) 0)
		 ((= (length step) 1)
		  (car step))
		 ;((and (calc/variable? ))
		 (t
		  (apply #'calc/sum step)))))
	  
	((calc/product? exp)
	 (let ((step (gen/filter (lambda (x) (not (eq 1 x)))
				 (map 'list #'calc/simplify (rest exp)))))
	   (cond ((gen/any (lambda (x) (eq 0 x)) step)
		  0)
		 ((= (length step) 1) (car step))
		 (t (apply #'calc/product step)))))
	(t exp)))



"
(calc/deriv (calc/sum (calc/product 'x 2) x) 'x)
"
(defparameter *calc/test* (calc/sum
			   (calc/product 'x 2)
			   12))
  
