(defun element-of-set-p (x set)
  (cond ((null set) nil)
        ((= x (entry set)) t)
        ((< x (entry set))
         (element-of-set-p
          x 
          (left-branch set)))
        ((> x (entry set))
         (element-of-set-p
          x 
          (right-branch set)))))

(defun adjoin-set (x set)
  (cond ((null set) (make-tree x '() '()))
        ((= x (entry set)) set)
        ((< x (entry set))
         (make-tree 
          (entry set)
          (adjoin-set x (left-branch set))
          (right-branch set)))
        ((> x (entry set))
         (make-tree
          (entry set)
          (left-branch set)
          (adjoin-set x (right-branch set))))))


(defun intersection-set (set1 set2)
  (cond ((or (null set1) (null set2))
	 '())
	((element-of-set-p (car set1) set2)
	 (cons (car set1)
	       (intersection-set (cdr set1)
				 set2)))
	(t (intersection-set (cdr set1)
				 set2))))
(defun union-set (set1 set2)
  (cond ((null set1)
	 set2)
	((null set2)
	 '())
	(t (union-set (cdr set1)
		      (adjoin-set (car set1) set2)))))

(defun entry (tree) (car tree))
(defun left-branch (tree) (cadr tree))
(defun right-branch (tree) (caddr tree))
(defun make-tree (entry left right)
  (list entry left right))

(defun tree->list-1 (tree)
  (if (null tree)
      '()
      (append 
       (tree->list-1 
        (left-branch tree))
       (cons (entry tree)
             (tree->list-1 
              (right-branch tree))))))

(defun tree->list-2 (tree)
  (labels ((copy-to-list (tree result-list)
	     (if (null tree)
		 result-list
		 (copy-to-list 
		  (left-branch tree)
		  (cons (entry tree)
			(copy-to-list 
			 (right-branch tree)
			 result-list))))))
    (copy-to-list tree '())))

(defun list->tree (elements)
  (car (partial-tree 
        elements (length elements))))

(defun partial-tree (elts n)
  (if (= n 0)
      (cons '() elts)
      (let* ((left-size 
               (floor (- n 1) 2))
	     
	     (left-result 
               (partial-tree 
                elts left-size))
	     
             (left-tree 
               (car left-result))
             (non-left-elts 
               (cdr left-result))
             (right-size 
               (- n (+ left-size 1)))
	     
	     (this-entry 
               (car non-left-elts))
             (right-result 
               (partial-tree 
                (cdr non-left-elts)
                right-size))
	     
             (right-tree 
               (car right-result))
             (remaining-elts 
               (cdr right-result)))
	
        (cons (make-tree this-entry 
                         left-tree 
                         right-tree)
              remaining-elts))))

(defparameter *test-tree* '(7 (3 (1) (5)) (9 () (11))))

"--deriv--"
(defun deriv (exp var)
  (cond ((numberp exp) 0)
        ((variablep exp)
         (if (same-variable-p exp var) 1 0))
        ((sump exp)
         (make-sum (deriv (addend exp) var)
                   (deriv (augend exp) var)))
        ((productp exp)
         (make-sum
          (make-product 
           (multiplier exp)
           (deriv (multiplicand exp) var))
          (make-product 
           (deriv (multiplier exp) var)
           (multiplicand exp))))
	((exponentiationp exp)
	 (make-product
	  (make-product
	  (exponent exp)
	  (make-exponentiation (base exp)
				(make-sum (exponent exp) -1)))
	   (deriv (base exp) var)))
        (t (error (format nil "unknown expression type: DERIV~%~a" exp)))))

(defun =numberp (exp num)
  (and (numberp exp)
       (= exp num)))

(defun variablep (e)
  (symbolp e))

(defun same-variable-p (v1 v2)
  (and (variable? v1)
       (variable? v2)
       (eq v1 v2)))

(defun sump (e)
  (and (consp e)
       (eq (car e) '+)))

(defun addend (e)
  (second e))

(defun augend (e)
  (cddr e))

(defun make-sum (&optional a1 &rest a2)
  (format t "~&! ~d ; ~d~%" a1 a2)
  (let* ((a-rest* (if (< (length a2) 2)
		     a2
		     (apply #'make-sum a2)))
	 (a-rest (cdr a-rest*))
	 (a2* (if (numberp a-rest)
		  a-rest
		  (addend a-rest))))
    (print "---")
    (print a1)
    (print a-rest)
    (print a2*)
    (cond ((null a1)
	   (print 1)
	   0)
	  ((null a2*)
	   (print 2)
	   a1)
	  ((sump a1)
	   (print 3)
	   (make-sum (addend a1)
		     (apply #'make-sum
			    (augend a1)
			    a-rest)))
	  ((=numberp a1 0)
	   (print 4)
	   a-rest)
	  ((=numberp a2* 0)
	   (print 5)
	   (apply #'make-sum
		  a1
		  (cdr a-rest)))
	  ((and (numberp a1)
		(numberp a2*))
	   (print 6)
	   (apply #'make-sum
		  (+ a1 a2*)
		  (cdr a-rest)))
	  ((numberp a2*)
	   (print 7)
	   (apply #'list '+ a2* a1 (cdr a-rest)))
	  (t
	   (print 8)
	   (apply #'list '+ a1 a-rest)))))


(defun test (&optional a1 &rest a2)
  (apply #'list '+ (car a2) a1 (cdr a2)))

(defun productp (e)
  (and (consp e)
       (eq (car e) '*)))

(defun multiplier (e)
  (second e))

(defun multiplicand (e)
  (cddr e))

(defun make-product (m1 m2)
  (cond ((or (=numberp m1 0)
	     (=numberp m2 0))
	 0)
	((=numberp m1 1) m2)
	((=numberp m2 1) m1)
	((and (numberp m1)
	      (numberp m2))
	 (* m1 m2))
	(t (list '* m1 m2))))

(defun exponentiationp (e)
  (and (consp e)
       (eq (car e) '^)))

(defun base (e)
  (second e))

(defun exponent (e)
  (third e))

(defun make-exponentiation (b e)
  (cond ((=numberp e 0) 1)
	((=numberp e 1) b)
	((=numberp b 1) 1)
	((and (numberp b)
	      (numberp e))
	 (expt b e))
	(t (list '^ b e))))

"---"
  
(defun memq (item x)
  (cond ((null x) nil)
	((eq item (car x)) x)
	(t (memq item (cdr x)))))

(defun equal? (x y)
  (cond ((and (symbolp x) (symbolp y))
	 (eq x y))
	((and (listp x) (listp y))
	 (and (equal? (car x) (car y))
	      (equal? (cdr x) (cdr y))))
	(t nil)))



"~~~Pictures~~~"

(defun make-vect (x y)
  (cons x y))

(defun xcor-vect (vect)
  (car vect))

(defun ycor-vect (vect)
  (cdr vect))

(defun add-vect (v1 v2)
  (make-vect (+ (xcor-vect v1)
		(xcor-vect v2))
	     (+ (ycor-vect v1)
		(ycor-vect v2))))

(defun sub-vect (v1 v2)
  (make-vect (- (xcor-vect v1)
		(xcor-vect v2))
	     (- (ycor-vect v1)
		(ycor-vect v2))))

(defun scale-vect (scalar v)
  (make-vect (* scalar
		(xcor-vect v))
	     (* scalar
		(ycor-vect v))))

(defun make-frame (origin edge1 edge2)
  (list origin edge1 edge2))

(defun origin-frame (frame)
  (first frame))

(defun edge1-frame (frame)
  (second frame))

(defun edge2-frame (frame)
  (third frame))

(defun frame-coord-map (frame)
  (lambda (v)
    (add-vect
     (origin-frame frame)
     (add-vect
      (scale-vect (xcor-vect v)
		  (edge1-frame frame))
      (scale-vect (ycor-vect v)
		  (edge2-frame frame))))))

(defun make-segment (start end)
  (cons start end))

(defun start-segment (segment)
  (car segment))

(defun end-segment (segment)
  (cdr segment))

(defun outline ()
  (segments->painter
   (list (make-segment
	  (make-vector 0 0)
	  (make-vector 1 0))
	 (make-segment
	  (make-vector 1 0)
	  (make-vector 1 1))
	 (make-segment
	  (make-vector 1 1)
	  (make-vector 0 1))
	 (make-segment
	  (make-vector 0 1)
	  (make-vector 0 0)))))

"----"

(defun accumulate (op initial sequence)
  (labels ((iter (sequence total)
	     (if (null sequence)
		 total
		 (iter (rest sequence)
		       (funcall op (first sequence) total)))))
    (iter sequence initial)))

(defun fold-left (op initial sequence)
  (labels ((iter (sequence total)
	     (if (null sequence)
		 total
		 (iter (rest sequence)
		       (funcall op total (first sequence))))))
    (iter sequence initial)))

(defun fold-right (op initial sequence)
  (labels ((iter (sequence total)
	     (if (null sequence)
		 total
		 (iter (rest sequence)
		       (funcall op (first sequence) total)))))
    (iter sequence initial)))

(defun accumulate-n (op init seqs)
  (if (null (car seqs))
      nil
      (cons (accumulate op init (acc-map #'first seqs))
	    (accumulate-n op init (acc-map #'rest seqs)))))

(defun enumerate-interval (low high)
  (if (> low high)
      nil
      (cons low
	    (enumerate-interval (1+ low) high))))

(defun enumerate-tree (tree)
  (cond ((null tree) nil)
        ((not (listp tree)) (list tree))
        (t (append 
               (enumerate-tree (car tree))
               (enumerate-tree (cdr tree))))))

(defun acc-map (function seq)
  (accumulate (lambda (x y) (cons (funcall function x) y))
	      nil (reverse seq)))
  
(defun acc-append (seq1 seq2)
  (accumulate #'cons seq2 (reverse-list seq1)))

(defun acc-length (seq)
  (accumulate #'(lambda (x y) (1+ y)) 0 seq))

(defun horner-eval (x coefficient-sequence)
  (accumulate #'(lambda (this-coeff higher-terms)
		  (+ this-coeff (* higher-terms x)))
	      0
	      (reverse coefficient-sequence)))

(defun count-leaves (tree)
  (accumulate #'(lambda (subtree total)
		  (+ total (length subtree)))
	      0
	      (acc-map #'enumerate-tree tree)))

(defun list-sum (list)
  (accumulate #'+ 0 list))

(defun flatmap (proc seq)
  (accumulate #'append nil (acc-map proc seq)))

(defun permutations (set)
  (if (null set)
      (list nil)
      (flatmap (lambda (x)
		 (map 'list
		      (lambda (p)
			(cons x p))
		      (permutations
		       (remove x set))))
	       set)))

(defun my-remove (item sequence)
  (filter (lambda (x) (not (= x item)))
	  sequence))

(defun unique-pairs (n)
  (flatmap (lambda (i)
	     (acc-map (lambda (j)
			(list i j))
		      (enumerate-interval 1 (1- i))))
	   (enumerate-interval 1 n)))

(defun ordered-k-lets (n k)
  (if (= k 2)
      (unique-pairs n)
      (flatmap (lambda (k)		 
		 (acc-map (lambda (j)
			    (append k (list j)))
			  (enumerate-interval 1 (1- (last-item k)))))
	       (ordered-k-lets n (1- k)))))

(defun last-item (list)
  (car (last list)))


(defun sum-to-p (s &rest nums)
  (= s (list-sum nums)))

(defun sum-triplets (n s)
  (filter (lambda (trip)
	    (apply #'sum-to-p s trip))
	  (ordered-k-lets n 3)))

"""
(defun queens (board-size)
  (labels ((empty-board ()
	     (list))
	   (safep (k positions)
	     )
	   (queen-cols (k)
	     (if (= k 0)
		 (list empty-board)
		 (filter
		  (lambda (positions)
		    (safe? k positions))
		  (flatmap
		   (lambda (rest-of-queens)
		     (map (lambda (new-row)
			    (adjoin-position
			     new-row
			     k
			     rest-of-queens))
			  (enumerate-interval
			   1
			   board-size)))
		   (queen-cols (1- k))))))
    (queen-cols board-size)))
"""
"-"
(defparameter *m* (list (list 1 2 3 4)
			(list 4 5 6 6)
			(list 6 7 8 9)))

(defun dot-product (v w)
  (accumulate #'+ 0 (map 'list #'* v w)))

(defun matrix-*-vector (m v)
  (acc-map (lambda (rida) (dot-product rida v)) m))

(defun transpose (mat)
  (accumulate-n #'cons
		nil
		(reverse mat)))

(defun matrix-*-matrix (m n)
  (let ((cols (transpose n)))
    (acc-map (lambda (rida) (matrix-*-vector (verbose cols)
					     (verbose rida)))
	     m)))

"""(defmacro unless! (pred &rest a)
  `(if (not ,pred) ,a))"""
"-"
"--------------"

(defun last-pair (list)
  (cond ((null (cdr list)) list)
	(t (last-pair (cdr list)))))

(defun reverse-list (list)
  (labels ((reverse-iter (list total)
	   (cond ((null list) total)
		 (t
		  (reverse-iter (rest list)
				(cons (first list)
				      total))))))
    (reverse-iter list nil)))

(defun deep-reverse (list)
  (labels ((reverse-iter (list total)
	   (cond ((null list) total)
		 (t
		  (reverse-iter (rest list)
				(cons
				 (if (listp (first list))
				     (deep-reverse (first list))
				     (first list))
				 total))))))
    (reverse-iter list nil)))

(defun filter (pred list)
  (labels ((filter-iter (pred list total)
	     (if (null list) total
		 (filter-iter pred (rest list)
			      (if (funcall pred (first list))
				  (cons (first list) total)
				  total)))))
    (filter-iter pred (reverse-list list) nil)))

(defun same-parity (a &rest n)
  (cons a
	(filter
	 (if (evenp a)
	     #'evenp
	     #'oddp)
	 n)))

(defun map-my (proc items)
  (labels ((map-iter (proc items total)
	     (if (null items)
		 total
		 (map-iter proc
			   (rest items)
			   (cons
			    (funcall proc (first items))
			    total)))))
    (map-iter proc (reverse-list items) nil)))

(defun tree-map (proc tree)
  (map-my (lambda (item)
	    (if (listp item)
		(tree-map proc item)
		(funcall proc item)))
	    tree))

(defun scale-list (items factor)
  (map-my #'(lambda (x) (* x factor))
	  items))

(defun for-each (proc items)
  (when (not (null items))
  (funcall proc (first items))
  (for-each proc (rest items))))

(defun fringe (list)
  (cond ((null list) nil)
	((atom list) (list list))
	(t
	 (append (fringe (car list))
		 (fringe (cdr list))))))
"

(defparameter *x* 
  (list (list 1 12) (list 3 4)))

(defun make-mobile (left right)
  (list left right))

(defun make-branch (length structure)
  (list length structure))

(defun left-branch (mobile)
  (car mobile))

(defun right-branch (mobile)
  (cadr mobile))

(defun branch-length (branch)
  (car branch))

(defun branch-structure (branch)
  (cadr branch))

(defun balancedp (mobile)
  (labels ((mobile-weight (mobile)
	     (let ((left-weight (eval-branch (left-branch mobile)))
		   (right-weight (eval-branch (right-branch mobile))))
	       (cond ((or (null left-weight)
			  (null right-weight)
			  (not (eq left-weight right-weight)))
		      nil)
		     (t (+ left-weight
			   right-weight)))))
	   (eval-branch (branch)
	     (let ((weight (if (listp (branch-structure branch))
			       (mobile-weight (branch-structure branch))
			       (branch-structure branch))))
	       (if (null weight)
		   nil
		   (* weight (branch-length branch))))))
    (atom (mobile-weight mobile))))
"

(defun square-tree (list)
  (tree-map #'square list))
(defun square (x)
  (* x x))

(defun subsets (s)
  (if (null s)
      (list nil)
      (let ((rest (subsets (cdr s))))
        (append rest (map-my #'(lambda (x) (cons (first s) x)) rest)))))



"--------"

(defun make-rat (n d)
  (let ((g (gcd n d)))
    (cond ((> (* n d) 0)
	   (cons (/ (abs n) g) (/ (abs d) g)))
	  (t
	   (cons (/ (- (abs n)) g) (/ (abs d) g))))))

(defun numer (x)
  (car x))
(defun denom (x)
  (cdr x))
(defun add-rat (x y)
  (make-rat
   (+ (* (numer x) (denom y))
      (* (numer y) (denom x)))
   (* (denom x) (denom y))))

(defun sub-rat (x y)
  (make-rat
   (- (* (numer x) (denom y))
      (* (numer y) (denom x)))
   (* (denom x) (denom y))))

(defun mul-rat (x y)
  (make-rat
   (* (numer x) (numer y))
   (* (denom x) (denom y))))

(defun div-rat (x y)
  (make-rat
   (* (numer x) (denom y))
   (* (denom x) (numer y))))

(defun equal-ratp (x y)
  (= (* (numer x) (denom y))
     (* (numer y) (denom x))))

(defun print-rat (rat)
  (format t "~a/~a" (numer rat) (denom rat)))

"-----------------"

(defun cube (x)
  (* x x x))

(defun deb (x)
  (format t "~a~%" x)
  x)
"""
(defun filtered-accumulate (combiner null-value term a next b filter)
  (labels ((iter (a result)
	   (if (> a b)
	       result
	       (iter (funcall next a)
		     (if (funcall filter a)
			 (funcall combiner result (funcall term a))
			 result)))))
    (iter a null-value)))


(defun accumulate (combiner null-value term a next b)
  (filtered-accumulate combiner null-value term a next b #'(lambda (x) t)))

(defun sum (term a next b)
  (accumulate #'+ 0 term a next b))

(defun product (term a next b)
  (accumulate #'* 1 term a next b))

(defun pi~ (n)
  (* 4.0
     (product #'(lambda (x) (if (oddp x)
				(/ (+ 1 x) (+ 2 x))
				(/ (+ 2 x) (+ 1 x))))
	      1
	      #'1+
	      n)))

(defun factorial (n)
  (product #'identity 1 #'1+ n))


(defun cont-frac (n d k)
  (labels ((iter (i present)
	     (if (= i 0)
		 present
		 (iter (1- i) (/ (funcall n i)
				 (+ (funcall d i) present))))))
    (iter k 0)))

(defun golden-ratio~ (k)
  (1+
   (cont-frac (lambda (i) 1.0)
	      (lambda (i) 1.0)
	      k)))

(defun e~ (k)
  (+ 2
     (cont-frac (lambda (i) 1.0)
		(lambda (i) (if (= (rem i 3) 2)
				(* 2 (ceiling i 3))
				1))
		k)))
(defun tan~ (x k)
  (cont-frac (lambda (i) (if (= i 1) x (* x x)))
	     (lambda (i) (- (* 2.0 i) 1))
	     k))

(defun fixed-point (f first-guess)
  (let ((tolerance 0.00001))
    (labels ((closep (v1 v2)
	       (< (abs (- v1 v2)) tolerance))
	     (try (guess)
	       (let ((next (funcall f guess)))
		 (if (closep guess next)
		     next
		     (try next)))))
      (try first-guess))))

(defun average-damp (f)
  (lambda (x) (/ (+ x (funcall f x)) 2)))

(defun deriv (g)
  (let ((dx 0.001))
    (lambda (x)
      (/ (- (funcall g (+ x dx)) (funcall g x))
	 dx))))

(defun newton-transform (g)
  (lambda (x)
    (- x (/ (funcall g x)
	    (funcall (deriv g) x)))))

(defun newtons-method (g guess)
  (fixed-point (newton-transform g) guess))

(defun sqrt~ (x)
  (newtons-method (lambda (y) (- (* y y) x))
		  1.0))

(defun cubic (a b c)
  (lambda (x) (+ (cube x) (* a x x) (* b x) c)))

(defun square (x)
  (* x x))


(defun !double (f)
  (lambda (x) (funcall f (funcall f x))))

(defun compose (f g)
  (lambda (x) (funcall f (funcall g x))))

(defun repeated (f n)
  (if (= n 1)
      (lambda (x) (funcall f x))
      (compose f (repeated f (1- n)))))

(defun smooth (f)
  (let ((dx 0.00001))
    (lambda (x) (/ (sum f
			(- x dx)
			(lambda (x) (+ x dx))
			(+ x dx))
		   3))))


"""
